<%--
  Copyright 2011 The Kuali Foundation Licensed under the Educational Community
  License, Version 2.0 (the "License"); you may not use this file except in
  compliance with the License. You may obtain a copy of the License at
  http://www.osedu.org/licenses/ECL-2.0 Unless required by applicable law or
  agreed to in writing, software distributed under the License is distributed
  on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
  express or implied. See the License for the specific language governing
  permissions and limitations under the License.
--%>

<%@ page language="java" contentType="text/html; charset=UTF-8" pageEncoding="UTF-8"%>

<%@ taglib prefix="c"   uri="http://java.sun.com/jsp/jstl/core" %>
<%@ taglib prefix="kme" uri="http://kuali.org/mobility" %>
<%@ taglib prefix="form" uri="http://www.springframework.org/tags/form"%>
<%@ taglib prefix="spring" uri="http://www.springframework.org/tags"%>
<%@ taglib prefix="fn" uri="http://java.sun.com/jsp/jstl/functions" %>

<spring:message code="push.title" 	var="title"/>
<spring:message code="push.messages" 	var="pMessages"/>
<spring:message code="push.message.emergency" 	var="mEmergency"/>
<spring:message code="push.message.clear" 	var="mClear"/>
<spring:message code="push.message.send" 	var="mSend"/>

<kme:page title="${title}" id="push-webapp" backButton="true" homeButton="true" cssFilename="push" jsFilename="push">
	<kme:content>
		<form:form action="${pageContext.request.contextPath}/push" commandName="push" data-ajax="false" method="post">
			<input type="hidden" name="id" value="${device.id}"/>
			<input type="hidden" name="sender" value="${cookie.currentNetworkId.value}"/>
			
			<kme:listView id="pushForm" filter="false" dataTheme="c" dataInset="false">

				<c:if test="${not empty device}">
					<kme:listItem>				
						<h3><spring:message code="push.message.recipients" />:</h3> 
						<p><strong><c:out value="${device.deviceName}" /></strong></p>
						<p class="wrap"><spring:message code="push.device.devid" /> <c:out value="${device.deviceId}" /></p>
					</kme:listItem>
				</c:if>
				
				<c:if test="${empty device}">
					<kme:listItem>
						<label for="recipientSelect"><spring:message code="push.message.recipients" />:</label>
						<select data-native-menu="false" id="recipientSelect" name="recipientSelect">
							<option value="nil" selected>Select Group Recipients</option>
							<c:forEach items="${recipients}" var="msg">
								<option value="${msg.value}">${msg.key}</option>
							</c:forEach>
						</select>
					</kme:listItem>
				</c:if>
			
				<kme:listItem>
					<label for="pushselect">${pMessages}:</label>
					<select data-native-menu="false" id="pushselect" name="pushselect" onchange="selectDidChange($('#pushselect option:selected').text(), $('#pushselect').val());$('#pushselect').selectmenu('close');return;">
						<option value="nil" selected>Select Pre-Defined Push</option>
						<c:forEach items="${stockPushMessages}" var="msg">
							<option value="${msg.value}">${msg.key}</option>
						</c:forEach>
					</select>
				</kme:listItem>
			
				<kme:listItem>
					<label for="title"><spring:message code="push.message.title" />:</label>
					<form:textarea path="title" cols="40" rows="8" class="required" />
					<form:errors path="title"/>
				</kme:listItem>

			   	<kme:listItem>
					<label for="message"><spring:message code="push.message.body" />:</label>
					<form:textarea path="message" cols="40" rows="8" class="required" />
					<form:errors path="message"/>
				</kme:listItem>

			   	<kme:listItem>
					<label for="url"><spring:message code="push.message.url" />:</label>
					<form:input path="url" cols="40" rows="8" placeholder="Optional (include 'http://')" class="required" />
					<form:errors path="url"/>
				</kme:listItem>

			   	<kme:listItem>
					<form:checkbox path="emergency" value="Y" label="${mEmergency}"/>
				</kme:listItem>

			   	<kme:listItem>
					<fieldset class="ui-grid-a">
							<div class="ui-block-a"><a href="${pageContext.request.contextPath}/push" data-role="button" data-theme="c" id="clear">${mClear}</a></div>
							<div class="ui-block-b">
								<%-- <input data-theme="a" class="submit" type="submit" value="${mSend}"/> --%>
								<input data-theme="a" class="submit" type="button" value="${mSend}" id="submitButton"/>
							</div>
					</fieldset>
			   	</kme:listItem>
			</kme:listView>
		</form:form>



				<script type="text/javascript">
					function onConfirm(buttonIndex) {
						if(buttonIndex == 2){
							$("#push").submit();
						}
					}
				
					$("#submitButton").click(function() {
						if($("#recipientSelect").val() == "all"){
							if($.cookie('native') == "yes"){
								navigator.notification.confirm(
										"You're attempting to send a Push Notification to ALL users. This should only be done for an emergency.",  // message
										function (buttonIndex) {
											if(buttonIndex == 2){
												$("#push").submit();
											}
										},				// callback to invoke with index of button pressed
										'CAUTION:',		// title
										'Cancel,Send'	// buttonLabels
									);
							}else{
								if(confirm("You're attempting to send a Push to ALL users. This should only be for an emergency.")){
									$("#push").submit();
								}
							}
						}else{
							$("#push").submit();
						}
					});
				</script>
<br/>

			<kme:listView id="pushDevices" filter="false" dataTheme="c" dataInset="true">
				<kme:listItem>				
					<a href=${pageContext.request.contextPath}/devices>
						<h3><spring:message code="push.registered.devices" /></h3>
						<p class="wrap"><spring:message code="push.registered.devices.subtitle" /></p>
					</a>
				</kme:listItem>
			</kme:listView>
			
			<kme:listView id="pushHistory" filter="false" dataTheme="c" dataInset="true">
				<kme:listItem>				
					<a href=${pageContext.request.contextPath}/push/history>
				  		<h3><spring:message code="push.history" /></h3>
				  		<p class="wrap"><spring:message code="push.history.subtitle" /></p>
				  	</a>
			   	</kme:listItem>
			</kme:listView>

			<kme:listView id="pushSenders" filter="false" dataTheme="c" dataInset="true">
				<kme:listItem>				
					<a href=${pageContext.request.contextPath}/pushsenders>
				  		<h3><spring:message code="push.senders" /></h3>
				  		<p class="wrap"><spring:message code="push.senders.subtitle" /></p>
				  	</a>
			   	</kme:listItem>
			</kme:listView>


	</kme:content>
</kme:page>
