<%--
Copyright 2014 The Kuali Foundation Licensed under the Educational
Community License, Version 2.0 (the "License"); you may not use this file
except in compliance with the License. You may obtain a copy of the License
at http://www.osedu.org/licenses/ECL-2.0 Unless required by applicable law or
agreed to in writing, software distributed under the License is distributed
on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
express or implied. See the License for the specific language governing
permissions and limitations under the License.
--%>
<%@ page language="java" contentType="application/javascript; charset=UTF-8" pageEncoding="UTF-8"%>
<%@ taglib prefix="c" uri="http://java.sun.com/jsp/jstl/core" %>
<%@ taglib prefix="spring" uri="http://www.springframework.org/tags"%>
<spring:message code="academics.title" var="msgAcademicsTitle"/>
<spring:message code="academics.scheduleOfClasses" var="msgAcademicsScheduleOfClasses"/>
<spring:message code="academics.classSchedule" var="msgAcademicsClassSchedule"/>
<spring:message code="academics.classSearch" var="msgAcademicsClassSearch"/>
<spring:message code="academics.gradeAlerts" var="msgAcademicsGradeAlerts"/>
<spring:message code="academics.searchResult" var="msgAcademicsSearchResult"/>

var search = angular.module("searchApp", ['ngRoute','ngSanitize','ui.bootstrap'],
	function($routeProvider,$locationProvider) {
		$locationProvider.html5Mode(false);
        $routeProvider.
        when('/', {
        	templateUrl: '${pageContext.request.contextPath}/academics/templates/searchDetails',           
        	controller: 'searchController'
        }).when('/searchResults', {
        	templateUrl: '${pageContext.request.contextPath}/academics/templates/searchResults', 
        	controller: 'searchResultsController'
        }).when('/searchResultDetails', {
        	templateUrl: '${pageContext.request.contextPath}/academics/templates/searchResultDetails', 
        	controller: 'searchResultDetailsController'
        }).otherwise({
        	redirectTo: "/"
        });
	}
);

search.factory('SearchData', function() {
	return {
        searchToolTitle:"<c:out value="${msgAcademicsClassSearch}"/>",
        searchPageTitle:null,
		data:null,
		class:null,
		arrayCheck: function(arrayToTest) {
			if(angular.isArray(arrayToTest)) {
				return arrayToTest;
			} else {
				return [arrayToTest];
			}
		}
	};
});

search.controller("searchResultDetailsController", function($scope,$http,$templateCache,$location,$sce,$log,SearchData) {
	$scope.init = function() {
        $scope.SearchData = SearchData;
		$scope.searchResultData = $scope.SearchData.class;
		$scope.showSection = true;
        SearchData.searchPageTitle = SearchData.class.subjectId + ' ' + SearchData.class.catalogNumber;
    }
	
	$scope.sectionClick = function() {
		$scope.showSection = true;
		$scope.showDescription = false;
		$scope.showDetails = false;
	}
	
	$scope.descriptionClick = function() {
		$scope.showSection = false;
		$scope.showDescription = true;
		$scope.showDetails = false;
	}
	
	$scope.detailsClick = function() {
		$scope.showSection = false;
		$scope.showDescription = false;
		$scope.showDetails = true;
	}
	
	$scope.isArray = function(isMeetings) {
		return angular.isArray(isMeetings);
	}
	
	$scope.callArrayCheck = function(arrayToTest) {
		console.log(arrayToTest);
		return $scope.SearchData.arrayCheck(arrayToTest);
	}
});

search.controller("searchResultsController", function($scope,$http,$templateCache,$location,$sce,$log,SearchData) {
	$scope.init = function() {
        SearchData.searchPageTitle = "${msgAcademicsSearchResult}";
		$scope.SearchData = SearchData;
		$scope.searchResult = $scope.SearchData.data;
	}
	
	$scope.callArrayCheck = function(arrayToTest) {
		return $scope.SearchData.arrayCheck(arrayToTest);
	}
	$scope.classDetails = function(selectedClass) {
		$scope.SearchData.class = selectedClass;
		$location.path('/searchResultDetails');
	}
});

search.controller("searchController", function($scope,$http,$templateCache,$location,$sce,$log,SearchData) {
	$scope.init = function() {
        SearchData.searchPageTitle = SearchData.searchToolTitle;
		$scope.SearchData = SearchData;
		$scope.buildTerm();
		$scope.buildSchool("ALL");

        var menuItems = "{\"menus\": ["+
                "{ \"url\":\"/academics\" , \"label\":\"<c:out value="${msgAcademicsTitle}"/>\" },"+
                "{ \"divider\":\"true\"},"+
                "{ \"url\":\"/academics/terms\" , \"label\":\"<c:out value="${msgAcademicsScheduleOfClasses}"/>\" },"+
                "{ \"url\":\"/myAcademics\" , \"label\":\"<c:out value="${msgAcademicsClassSchedule}"/>\" },"+
                "{ \"url\":\"/academics/search\" , \"label\":\"<c:out value="${msgAcademicsClassSearch}"/>\" },"+
                "{ \"url\":\"/myAcademics/gradeAlerts\" , \"label\":\"<c:out value="${msgAcademicsGradeAlerts}"/>\" },"+
                <%--"{ \"divider\":\"true\"},"+--%>
              <%--  "{ \"url\":\"/preferences\" , \"label\":\"Preferences\" }"+--%>
                "]}";

        $scope.menuItems = eval ("(" + menuItems + ")");
	}
	
	$scope.kmeNavLeft = function() {
        // send to home screen.
        window.history.back();
    }
	
	$scope.courseSearch = function() {
		var dataEntered = new Object();
		
		dataEntered.termId = $scope.termSelect;

		dataEntered.careerId = $scope.schoolSelect;
		
		dataEntered.subjectId = $scope.subjectSelect;
		
		if (angular.isUndefined($scope.courseNumber)) {
			dataEntered.catalogNumber = "";
		} else {
			dataEntered.catalogNumber = $scope.courseNumber;
		}
		
		if (angular.isUndefined($scope.instructorName)) {
			dataEntered.instructor = "";
		} else {
			dataEntered.instructor = $scope.instructorName;
		}

        dataEntered.filterCriteria = [];
        if ($scope.oneLevel) {
			dataEntered.filterCriteria.push("100");
		}

        if ($scope.twoLevel) {
            dataEntered.filterCriteria.push("200");
        }

		if ($scope.threeLevel) {
            dataEntered.filterCriteria.push("300");
		}

		if ($scope.fourLevel) {
            dataEntered.filterCriteria.push("400");
		}
		
		if ($scope.fiveLevelPlus) {
            dataEntered.filterCriteria.push("500");
		}

	    dataEntered.showOpen = $scope.openClasses;

		$http({
			method: 'POST',
			url: '<c:out value="${pageContext.request.contextPath}"/>/services/cxfacademics/searchClasses',
			data: dataEntered,
			headers: {'Content-Type': 'application/x-www-form-urlencoded'}
		}).success(function(data, status, headers, config) {
			if( status != 200 ) {
                console.log("failure number was " + status)
            }
            $scope.SearchData.data = data.searchResult;
            $location.path('/searchResults');
            
		}).error(function(data, status, headers, config) {
			console.log("we got problems");
		});
	}
	
	$scope.buildTerm = function() {
		$http({
            method: 'GET',
            url: '<c:out value="${pageContext.request.contextPath}"/>/services/cxfacademics/getTerms'
        }).success(function(data, status, headers, config) {
            if( status != 200 ) {
                console.log("failure number was " + status)
            }
            $scope.terms = data.term;
            $scope.termSelect = $scope.terms[0].id;
            $scope.buildSubject($scope.terms[0].id,"ALL");
        }).error(function(data, status, headers, config) {
            console.log("we got problems");
        });
	}
	
	$scope.termChange = function() {
		$scope.buildSchool($scope.termSelect);
	}
	
	$scope.buildSchool = function(term) {
		var anySubjects = new Object();
		anySubjects.id = "ALL";
		anySubjects.description = "Any";
		var data = new Object();
		data.termId = term;
		$http({
			method: 'POST',
			url: '<c:out value="${pageContext.request.contextPath}"/>/services/cxfacademics/getCareers',
			data: data,
			headers: {'Content-Type': 'application/x-www-form-urlencoded'}
		}).success(function(data, status, headers, config) {
			var allCareers = data.academicCareer;
			allCareers.unshift(anySubjects);
			$scope.schools = allCareers;
			$scope.schoolSelect = $scope.schools[0].id;
		}).error(function(data, status, headers, config) {
			console.log("we got problems");
		});
	}
	
	$scope.schoolChange = function() {
		$scope.buildSubject($scope.termSelect, $scope.schoolSelect);
	}
	
	$scope.buildSubject = function(term, school) {
		var anySubjects = new Object();
		anySubjects.id = "ALL";
		anySubjects.description = "Any";
		var data = new Object();
		data.termId = term;
		data.careerId = school;
		$http({
			method: 'POST',
			url: '<c:out value="${pageContext.request.contextPath}"/>/services/cxfacademics/getSubjects',
			data: data,
			headers: {'Content-Type': 'application/x-www-form-urlencoded'}
		}).success(function(data, status, headers, config) {
			var allSubjects = data.subject;
			allSubjects.unshift(anySubjects);
			$scope.subjects = allSubjects;
			$scope.subjectSelect = $scope.subjects[0].id;
		}).error(function(data, status, headers, config) {
			console.log("we got problems");
		});
	}
	
	$scope.subjectChange = function() {
		<%-- place holder for if something is added for changing the subject drop down --%>
	}
});